      PROGRAM gaueli  !  Gaussian elimination
c
c     g77 -O gaueli.f       OR    xlf -O gaueli.f
c     ./a.exe 20 1.d-11 1
c
      implicit double precision (a-h,o-z)
      parameter (nsize=500)
      dimension a(nsize,nsize),x(nsize),y(nsize),ipvt(nsize)
      character*80 arg
c
      call getarg(1,arg)
      read(arg,*) n
      call getarg(2,arg)
      read(arg,*) eps
      call getarg(3,arg)
      read(arg,*) idata
c
      if(idata.eq.0) then
        call setfrank (a,nsize,n)
      else
        call setrandm (a,nsize,n)
      endif
c
      do i=1,n
        y(i)=1.0
        x(i)=0
      enddo
      call dmvmul (a,nsize,n,n,y,x)
c
      call dgauss (a,nsize,n,ipvt)
      do i=1,n
        write(*,*) "ipvt(",i,")=",ipvt(i)
      enddo
      call dlusol (a,nsize,n,x,ipvt)
      write(*,*) (x(i),i=1,n)
c
       do i=1,n
        v=1.0
        derr=abs(v-x(i))
        if(derr.gt.eps) write(*,*) "Error i=",i," diff=",derr
       enddo
c
      END
      subroutine dgauss (a,lda,n,ipvt)
      implicit double precision (a-h,o-z)
      dimension a(lda,n),ipvt(n)
      do K=1,n-1
        l=idamax(n-k+1,a(k,k),1)+k-1
        ipvt(k)=l
        call dswap(n,a(k,1),lda,a(l,1),lda)      ! FCS
c       call dswap(n-k+1,a(k,k),lda,a(l,k),lda)  ! PCS
        call dscal(n-k,1.d0/a(k,k),a(k+1,k),1)
        do J=k+1,n
           do I=k+1,n
             a(i,j)=a(i,j)-a(i,k)*a(k,j)
           enddo
        enddo
      enddo
      return
      END
      SUBROUTINE dlusol (a,lda,n,x,ipvt)  ! Substitution 
      implicit double precision (a-h,o-z)
      dimension a(lda,n),x(n),ipvt(*)
c
      do j=1,n-1     ! Permutation back (FCS)
        t=x(j)             !  FCS
        x(j)=x(ipvt(j))    !  FCS
        x(ipvt(j))=t       !  FCS
      enddo               !  FCS
c
      do j=1,n-1     ! Forward substitution
         do i=j+1,n
          x(i)=x(i)-a(i,j)*x(j)
         enddo
      enddo
c
c     write(*,*) ' forward solv end'
c     write(*,*) (x(i),i=1,n)
c
      do i=n,1,-1     ! Backward substitution 
        x(i)=x(i)/a(i,i)
        do k=1,i-1
          x(k)=x(k)-x(i)*a(k,i)
        enddo
      enddo
c
c     write(*,*) ' backward solv end'
c     write(*,*) (x(i),i=1,n)
c
      RETURN
      END
      SUBROUTINE dmvmul (a,lda,m,n,x,y)
      double precision a(lda,n),x(*),y(*)
       do I=1,m
         do J=1,n
          y(i)=y(i)+A(i,J)*X(J)  
         enddo
       enddo
      return
      END
      SUBROUTINE dmtmul (a,lda,m,n,x,y)
      double precision a(lda,n),x(*),y(*)
       do J=1,n
         do I=1,m
          y(i)=y(i)-a(i,j)*x(j)  
         enddo
       enddo
      return
      END
      subroutine matprn(a,lda,n)
      implicit double precision (a-h,o-z)
      dimension a(lda,n)
c
       do i=1,n
        write(*,1) (a(i,j),j=1,n)
    1   format (11f6.3)
       enddo
c
      RETURN
      END
      SUBROUTINE setfrank (a,lda,n)        !  set Frank matrix of order n
      double precision a(lda,n)
       do j=1,n
         do i=1,n
          a(i,j)=n-max(i,j)+1
c         a(i,j)=max(i,j)
         enddo
       enddo
      return
      END
      SUBROUTINE setrandm (a,lda,n)        !  set Ramdom matrix of order n
      double precision a(lda,n)
c     call srand48(68111)                  !  dranf2c.c
      integer*8 iseed
      iseed=68111
       do j=1,n
         do i=1,n
          a(i,j)=ddrand(iseed)
         enddo
       enddo
      return
      END
      DOUBLE PRECISION FUNCTION ddrand(iseed)
      implicit double precision (a-h,o-z)
      integer*8 iseed,k,iirand
      k=iirand(iseed)
      ddrand=dble(dble(k)/dble(2147483647))
      return
      end
      FUNCTION iirand(iseed) ! multiplicative congruential generator (I+1) <= (I)*A mod(2^31-1)
      integer*8 iseed,k,m,iirand 
*
*     m=2**31-1   !  Warning in GNU (Integer at (^) too large except as negative number)
      k=iseed*16807
c     iseed=mod(k,m)
      iseed=mod(k,2147483647)
      iirand=iseed
*
      return
      end
      function idamax (n,x,ldx)
      double precision x(ldx,n),t
      t=abs(x(1,1))
      ix=1
       do j=2,n 
        if(abs(x(1,j)).gt.t) then
         ix=j
         t=abs(x(1,j))
        endif
       enddo  
      idamax=ix 
      return
      end
      subroutine dscal (n,a,x,ldx)
      double precision x(ldx,n),a
       do j=1,n    
        x(1,j)=a*x(1,j)
       enddo
      return
      end
      subroutine dswap (n,x,ldx,y,ldy)                         
      double precision x(ldx,n),y(ldy,n),t
       do j=1,n 
        t=x(1,j)
        x(1,j)=y(1,j)
        y(1,j)=t
       enddo
      return
      end
      subroutine daxpy (n,a,x,ldx,y,ldy)
      double precision x(ldx,n),y(ldy,n),a                   
       do j=1,n
        y(1,j)=y(1,j)+a*x(1,j)
       enddo
      return 
      end

