/******************************************************************************
*      basic procedure for CT3D solver                                        *
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "ct3d.h"
/*** external functions ***/
#include "LIButility.h"
/*************************
     create dofs table
**************************/
void create_doftab( FILE* fp_out, FILE* fp_log, CT3D *ct3d)
{
	int i,j,ii,jj,kk;
	int ncon,nfreedom;
/** Allocate ***/
	ct3d->doftab=vectorI( NDOF * (ct3d->numnp) );
/*** set fill 0 ***/	
	for(i=0;i< ct3d->numnp;i++){
		for(j=0;j< NDOF;j++){
			(ct3d->doftab)[ NDOF*i+j]=0;
		}
	}
/*** set fill 1 at used dof ***/
	for(i=0;i< ct3d->numnp;i++){
		for(j=0;j<NDOF;j++){
			(ct3d->doftab)[NDOF*i+j]=1;
		}
	}
/*** set spc ***/	
	ncon=0;
	for(i=0;i< ct3d->nspc;i++){
		ii =(ct3d->spc)[i].node_id;
		jj =(ct3d->spc)[i].dof_id;
		kk =NDOF*(ii-1)+jj;
		ncon++;
		(ct3d->doftab)[kk-1]=-ncon;
	}
/*** alive node***/
	nfreedom=0;
	for(i=0;i< NDOF * (ct3d->numnp) ;i++){
		if( (ct3d->doftab)[i] == 1 ) {
			(ct3d->doftab)[i]=++nfreedom;
		}
	}
/*** show table***/
#if DEBUG
	fprintf(fp_log,"### DOFTAB List ####\n");
	for(i=0;i< ct3d->numnp;i++){
		fprintf(fp_log,"#node id=%d",i+1);
		for(j=0;j<NDOF;j++){
			fprintf(fp_log," %d ",(ct3d->doftab)[NDOF*i+j]);
		}
		fprintf(fp_log,"\n");
	}
#endif
	fprintf(fp_out,"### number of freedoms=%d\n",nfreedom);
	fprintf(fp_out,"### number of constrain=%d\n",ncon);
	ct3d->nfreedom=nfreedom;
	ct3d->ncon=ncon;
/*** message **/
	fprintf( fp_log,"### create_doftab FINISHED!\n");
	fflush( fp_log );
}
/***************************************
	Create_ConstList for X2D
***************************************/
void create_constList(FILE* fp_out,FILE* fp_log, CT3D* ct3d)
{
	int i,ii;
	int id,id1;
	double value;
/***************************/
/*** Allocation     ********/
/***************************/
	if( (ct3d->ncon) > 0 ){
		(ct3d->cval )=vectorR( ct3d->ncon );
	}
/***************************/
/*** Spc Loop       ********/
/***************************/
	for(i=0;i<ct3d->nspc;i++){
		id     =(ct3d->spc)[i].node_id;
		id1    =(ct3d->spc)[i].dof_id;
		value  =(ct3d->spc)[i].value;
		if( (ct3d->doftab) [ NDOF*(id-1)+id1-1 ] < 0 ){
			ii=-(ct3d->doftab) [ NDOF*(id-1)+id1-1 ];
			(ct3d->cval )[ii-1]=value;
		}
	}
/***************************/
/*** Show           ********/
/***************************/
	fprintf(fp_out,"#### Number of nodes            = %d\n",ct3d->numnp);
	fprintf(fp_out,"#### Number of freedoms         = %d\n",ct3d->nfreedom);
	fprintf(fp_out,"#### Number of Constrained DOFs = %d\n",ct3d->ncon);
	fprintf(fp_log,"#### Number of nodes            = %d\n",ct3d->numnp);
	fprintf(fp_log,"#### Number of freedoms         = %d\n",ct3d->nfreedom);
	fprintf(fp_log,"#### Number of Constrained DOFs = %d\n",ct3d->ncon);
	fflush( fp_out );
	fflush( fp_log );
/*** message **/
	fprintf( fp_log,"### create_constList FINISHED!\n");
	fflush( fp_log );
}
/*************************
 find size of band matrix
**************************/
int find_bw( FILE *fp_out, FILE *fp_log, CT3D *ct3d)
{
	int i,nn,ls,j,k,me;
	int nid[NNOD],nd[NNOD*NDOF];
	int mht;
	int head;
/* clear height */
	mht=0;
/*** truss element *****/
	nn=2;
	for(i=0;i< ct3d->nume;i++){
		for(j=0;j<nn;j++){
			nid[j]=(ct3d->truss)[i].id[j];
		}
/*** Set dofs  per node ****/
		head=0;
		for(j=0;j<nn;j++){
			for(k=0;k<NDOF;k++){
				nd[ head + k]=(ct3d->doftab)[NDOF*(nid[j]-1)+k];
			}
			head+=NDOF;
		}
		ls= (ct3d->numnp) *NDOF;
		for(j=0;j< head ;j++){
			if(nd[j]>0){
				if(nd[j] < ls) ls=nd[j];
			}
		}
		for(j=0;j< head ;j++){
			if(nd[j]>0){
				me=nd[j]-ls+1;
				if( me > mht ) mht=me;
			}
		}
	}
	fprintf(fp_out,"*** B A N D    M A T R I X   I N F O R M A T I O N ****\n");
	fprintf(fp_out,"#### Number of subdiagonals %d\n",mht-1);
	fprintf(fp_log,"*** B A N D    M A T R I X   I N F O R M A T I O N ****\n");
	fprintf(fp_log,"#### Number of subdiagonals %d\n",mht-1);
/*** message **/
	fprintf( fp_log,"### find_bw FINISHED!\n");
	fflush( fp_log );
	return mht-1;
}
