/******************************************************
*  CT3D Client main                                   *
*******************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "ct3d.h"
/************************/
/** External Functions **/
/************************/
#include "Dataio.h"
#include "Basic.h"
#include "solveSTATIC.h"
#include "Cpu.h"
/**********************
	CT3D entry
***********************/
int ct3d_main(
	char inpfile[],
	char outfile[],
	char logfile[],
	char vtffile[],
	char pltfile[],
	CT3D* ct3d)
{
	FILE  *fp_inp;
	FILE  *fp_out;
	FILE  *fp_log;
	FILE  *fp_vtf;
	FILE  *fp_plt;
	float cputime[20]; /*** CPU counter ***/
/** Starting ***/
	cputime[0]=cpu_count();
/***** Log file *****/
	if( !strcmp(logfile,"") ) {
		fp_log=stdout;
	}else{
		if( (fp_log=fopen(logfile,"w")) == NULL){
			printf("log file cannot be opened!\n");
			return 1;
		}
	}
/***** Inp file *****/
	if( (fp_inp=fopen(inpfile,"r")) == NULL){
		fprintf(fp_log,"input file cannot be opened!\n");
		return 1;
	}
/***** Out file *****/
	if( (fp_out=fopen(outfile,"w")) == NULL){
		fprintf(fp_log,"output file cannot be opened!\n");
		return 1;
	}
/***** Vtf file *****/
	if( (fp_vtf=fopen(vtffile,"w")) == NULL){
		fprintf(fp_log,"vtf file cannot be opened!\n");
		return 1;
	}
/***** Vtf file *****/
	if( (fp_plt=fopen(pltfile,"w")) == NULL){
		fprintf(fp_log,"plt file cannot be opened!\n");
		return 1;
	}
/*** Message ***/
	fprintf(fp_log,"### STARTED!\n");
/** read input data **/
	read_data(fp_inp,fp_log,ct3d);
	fclose(fp_inp);
/** input data echo **/
	input_echo( fp_out, fp_log,ct3d);
	cputime[1]=cpu_count();
/** create dof table ***/
	create_doftab( fp_out, fp_log, ct3d);
/** create constList ***/
	create_constList( fp_out, fp_log, ct3d);
	cputime[2]=cpu_count();
/******************************/
/**** 3D Truss Analysis *******/
/******************************/
	solveSTATIC( fp_out, fp_log, fp_vtf, fp_plt, ct3d);
	cputime[3]=cpu_count();
	fclose( fp_vtf );
/****************************** 
  List of cpu time
*******************************/
	fprintf(fp_out,"#### CPU time informations (sec)\n");
	fprintf(fp_out," Input Data Phase..................%.2f\n",cputime[1]-cputime[0]);
	fprintf(fp_out," Preprocessing for Solver..........%.2f\n",cputime[2]-cputime[1]);
	fprintf(fp_out," Solver Process....................%.2f\n",cputime[3]-cputime[2]);
	fprintf(fp_out,"---------------------------------------\n"); 
	fprintf(fp_out," total.............................%.2f\n",cputime[3]-cputime[0]);
	fprintf(fp_log,"#### CPU time informations (sec)\n");
	fprintf(fp_log," Input Data Phase..................%.2f\n",cputime[1]-cputime[0]);
	fprintf(fp_log," Preprocessing for Solver..........%.2f\n",cputime[2]-cputime[1]);
	fprintf(fp_log," Solver Process....................%.2f\n",cputime[3]-cputime[2]);
	fprintf(fp_log,"---------------------------------------\n"); 
	fprintf(fp_log," total.............................%.2f\n",cputime[3]-cputime[0]); 
	fprintf(fp_log,"### FINISHED!\n");
	fclose(fp_out);
	fclose(fp_log);
	return 0;
}
