/******************************************************************************
*         data io module for CT3D  solver                                     *
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "ct3d.h"
#include "LIButility.h"
/*********************
  read input data     
**********************/
void read_data(FILE* fp_inp, FILE* fp_log, CT3D* ct3d)
{
	int i;
	CORD cord;
/***********************/   
/** read control data **/
/***********************/
	fgets(ct3d->title,NAME_LENG ,fp_inp);
	squeeze(ct3d->title,'\n');
/** General **/
	fscanf(fp_inp,"%d",&(ct3d->numnp));
	fscanf(fp_inp,"%d",&(ct3d->nume) );
	fscanf(fp_inp,"%d",&(ct3d->numat));
	fscanf(fp_inp,"%d",&(ct3d->nsect));
	fscanf(fp_inp,"%d",&(ct3d->nspc) );
	fscanf(fp_inp,"%d",&(ct3d->ncload));
/**********************/
/** read node data   **/
/**********************/
	if ( ( ct3d->node =(NODE *)malloc( ct3d->numnp * sizeof(NODE) ) ) == NULL ) {
		fprintf(fp_log,"Error:Memory is not enough for node\n");
		exit(1);
	}
	for(i=0;i< ct3d->numnp;i++){
		fscanf(fp_inp,"%lf %lf %lf",&cord.x,&cord.y,&cord.z);
		(ct3d->node)[i].cord=cord;
	}
/***********************/
/** read element data **/
/***********************/
/* truss */
	if(  ct3d->nume > 0) {
		if ( ( ct3d->truss=(TRUSS *)malloc( ct3d->nume * sizeof(TRUSS) ) ) == NULL ) {
			printf("Error:Memory is not enough for truss\n");
			exit(1);
		}
		for(i=0;i< ct3d->nume;i++){
			fscanf(fp_inp,"%d %d %d" ,
			&(ct3d->truss)[i].sid,
			&(ct3d->truss)[i].id[0],&(ct3d->truss)[i].id[1]);
		}
	}
/***********************/
/* read material data  */
/***********************/
	if ( ( ct3d->material=(MATERIAL *)malloc( ct3d->numat * sizeof(MATERIAL) ) ) == NULL ) {
		fprintf(fp_log,"Error:Memory is not enough for material\n");
		exit(1);
	}
	for( i=0;i< ct3d->numat;i++){
		fscanf(fp_inp,"%lf",
		&(ct3d->material)[i].young);
	} 
/***********************/
/* read section data  */
/***********************/
	if ( ( ct3d->section=(SECTION *)malloc( ct3d->nsect * sizeof(SECTION) ) ) == NULL ) {
		fprintf(fp_log,"Error:Memory is not enough for section\n");
		exit(1);
	}
	for( i=0;i< ct3d->nsect;i++){
		fscanf(fp_inp,"%lf %d",
		&(ct3d->section)[i].area,
		&(ct3d->section)[i].mid);
	} 
/*********************/
/* read spc data     */
/*********************/
	if( ct3d->nspc > 0) {
		if ( ( ct3d->spc=(SPC *)malloc( ct3d->nspc * sizeof(SPC) ) ) == NULL ) {
			fprintf(fp_log,"Error:Memory is not enough for SPC\n");
			exit(1);
		}
		for(i=0;i<ct3d->nspc;i++){
			fscanf(fp_inp,"%d %d %lf",
			&(ct3d->spc)[i].node_id,
			&(ct3d->spc)[i].dof_id,
			&(ct3d->spc)[i].value);
		}
	}
/*******************/
/* read cload data */
/*******************/
	if( ct3d->ncload > 0) {
		if ( ( ct3d->cload=(CLOAD *)malloc( ct3d->ncload * sizeof(CLOAD) ) ) == NULL ) {
			fprintf(fp_log,"Error:Memory is not enough for cload\n");
			exit(1);
		}
		for(i=0;i<ct3d->ncload;i++){
			fscanf(fp_inp,"%d %d %lf",
				&(ct3d->cload)[i].node_id,
				&(ct3d->cload)[i].dof_id,
				&(ct3d->cload)[i].value);
		}
	}
/*** message **/
	fprintf( fp_log,"### read_data FINISHED!\n");
	fflush( fp_log );
}
/************************
  input data echo
*************************/
void input_echo(FILE *fp, FILE *fp_log,CT3D* ct3d)
{
	CORD cord;
	int i,ielem;
	int nid;
	fprintf(fp,"///////////////////////////////////////////////\n");
	fprintf(fp,"// CT3D Solver Ver.1.1 (Band Matrix Version) //\n");
	fprintf(fp,"///////////////////////////////////////////////\n");
	fprintf(fp,"VERSION %f\n",VERSION);
#if LAPACK
	fprintf(fp,"Using LAPACK\n");
#else
	fprintf(fp,"Using SAMUKAWA Solver\n");
#endif
#if LOWER
	fprintf(fp,"Storing Lower Triangle Matrix in Band form\n");
#else
	fprintf(fp,"Storing Upper Triangle Matrix in Band form\n");
#endif
	fprintf(fp,"**** control data *****\n");
	fprintf(fp," Title:%s\n",ct3d->title);
	fprintf(fp," number of nodes          (numnp    )=%d\n",ct3d->numnp);
	fprintf(fp," number of truss elements (nume     )=%d\n",ct3d->nume );
	fprintf(fp," number of materials      (numat    )=%d\n",ct3d->numat);
	fprintf(fp," number of sections       (nsect    )=%d\n",ct3d->nsect);
	fprintf(fp," number of spc            (nspc     )=%d\n",ct3d->nspc);
	fprintf(fp," number of cload          (ncload   )=%d\n",ct3d->ncload);
	fprintf(fp,"**** node data *****\n");
	for(i=0;i<ct3d->numnp;i++){
		cord=(ct3d->node)[i].cord;
		fprintf(fp,"%d : %e %e %e\n",i+1,cord.x,cord.y,cord.z);
	}
	fprintf(fp,"**** element data *****\n");
	ielem=0;
	if( ct3d->nume > 0) {
		fprintf(fp,"**** truss element ****\n");
		for(i=0;i<ct3d->nume;i++){
			fprintf(fp,"%d :secid=%d:%d %d\n",
			++ielem,
			(ct3d->truss)[i].sid,
			(ct3d->truss)[i].id[0],
			(ct3d->truss)[i].id[1]);
		}
	}
	fprintf(fp,"***** material properties\n");
	for(i=0;i<ct3d->numat;i++){
		fprintf(fp,"matid    =%d:\n",i+1);
		fprintf(fp,"young    =%e\n",(ct3d->material)[i].young);
	}
	fprintf(fp,"***** section properties\n");
	for(i=0;i<ct3d->nsect;i++){
		fprintf(fp,"secid    =%d:\n",i+1);
		fprintf(fp,"area     =%e\n",(ct3d->section)[i].area);
		fprintf(fp,"matid    =%d\n",(ct3d->section)[i].mid);
	}
	if( ct3d->nspc > 0){
		fprintf(fp,"**** spc ****\n");
		for(i=0;i< ct3d->nspc;i++){
			nid= (ct3d->spc)[i].node_id;
			fprintf(fp,"SPC[%d]:nid=%d dof=%d val=%e\n",
			i+1,
			(ct3d->spc)[i].node_id,
			(ct3d->spc)[i].dof_id,
			(ct3d->spc)[i].value);
		}
	}
	if( ct3d->ncload > 0){
		fprintf(fp,"**** cload data *****\n");
		for(i=0;i< ct3d->ncload;i++){
			fprintf(fp,"%d : node=%d dof=%d val=%e\n",
				i+1,
				(ct3d->cload)[i].node_id,
				(ct3d->cload)[i].dof_id,
				(ct3d->cload)[i].value);
		}
	}
/*** message **/
	fprintf( fp_log,"### input_echo FINISHED!\n");
	fflush( fp_log );
}
