/*******************************************************
*  Solver STATIC                                       *
*******************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "ct3d.h"
/*** Basic  ***/
#include "Basic.h"
/*** Utility ***/
#include "LIButility.h"
/*** Truss Elements ***/
#include "LIBt3.h"
/*** VTF output **/
#include "writeVTF.h"
/*** static variables ***/
static int numnp;
static int nume;
static int numat;
static int nsect;
static int nspc;
static int ncload;
static NODE  *node;/**** Node Object ****/
static TRUSS *truss;/**** Truss element Object ****/
static MATERIAL *material;/**** material Object ****/
static SECTION  *section;/**** section Object ****/
static SPC   *spc;  /**** spc Object ****/
static CLOAD *cload;/**** cload Object ****/
static int nfreedom;/*** number of freedoms ***/
static int ncon;/*** number of constraint dofs ***/
static int *doftab;/*** doftable ***/
static double *tempx;/***  solution vector in condensed form ***/
static double *temp;/**** solution vector ****/
static double *cval;/** constrained value ***/
static RESULT_NODE    *result_node;/*** results node ***/
static RESULT_ELEMENT *result_element;/*** results element ***/
/*** static functions ***/
static void set_load();
static void assemble(FILE*);
static void add_gmat(int*,double*,int);
static void set_temp();
static void show_displacement(FILE*);
static void show_rforce(FILE*);
static void add_force(FILE*,int,double*);
static void show_stress(FILE*);
static void writePLT(FILE*,FILE*,CT3D*);
/*** static variable ***/
static int kb;/** size of Number of subdiagonals **/
static double *band;/** band marix **/
/** DPBSV Interface **/
#if LOWER
static char uplo='L';
#else
static char uplo='U';
#endif
static int  nrhs=1;
static int  ld_band;
static int  ld_rhs;
static int  info;
/*** LAPACK DPBSV Solver  **/
extern void dpbsv_();
/*** Prof. Samukawa's solver **/
/** for Lower Triangle Matrix **/
extern void dbandc_(double*, int*, int*, double*);
extern void dbandsc_(double*, int*, int*, double*);
/** for Upper Triangle Matrix **/
extern void dbandtc_(double*, int*, int*);
extern void dbandtsc_(double*, int*, int*, double*);
/*******************************
  static solver
********************************/
void solveSTATIC( FILE *fp_out, FILE *fp_log, FILE *fp_vtf, FILE *fp_plt, CT3D *ct3d)
{
	double *work;
	fprintf( fp_out,"######################\n" );
	fprintf( fp_out,"#### Static Analysis\n" );
	fprintf( fp_out,"######################\n" );
/**** Allocate array ***/ 
	ct3d->temp=vectorR( NDOF * (ct3d->numnp) );
	ct3d->tempx=vectorR( ct3d->nfreedom);
	if ( ( ct3d->result_node=(RESULT_NODE * )malloc( (ct3d->numnp) * sizeof( RESULT_NODE ) ) ) == NULL ) {
		fprintf(fp_log,"Error:Memory is not enough! a in result_node \n");
		exit(1);
	}
	if ( ( ct3d->result_element=(RESULT_ELEMENT * )malloc( (ct3d->nume) * sizeof( RESULT_ELEMENT ) ) ) == NULL ) {
		fprintf(fp_log,"Error:Memory is not enough! a in result_element \n");
		exit(1);
	}
/**** Set static variables ***/
	numnp =ct3d->numnp;
	nume  =ct3d->nume;
	numat =ct3d->numat;
	nsect =ct3d->nsect;
	nspc  =ct3d->nspc;
	ncload=ct3d->ncload;
	node =ct3d->node;
	truss=ct3d->truss;
	material=ct3d->material;
	section=ct3d->section;
	spc=ct3d->spc;
	cload=ct3d->cload;
	nfreedom=ct3d->nfreedom;
	ncon=ct3d->ncon;
	doftab=ct3d->doftab;
	cval=ct3d->cval;
	result_node   =ct3d->result_node;
	result_element=ct3d->result_element;
/** size of band matrix **/
	kb=find_bw(fp_out,fp_log,ct3d);
/** allocate bandmatrix **/
	band=vectorR( (kb+1)* nfreedom );
/** work **/
	tempx=ct3d->tempx;
	temp =ct3d->temp;
/** create load vector ***/   
	set_load( fp_log );
/** create & assemble stiffness matrix **/
	assemble( fp_log );
/** Solve Band Matrix **/
	ld_band=kb+1;
	ld_rhs=nfreedom;
#if LAPACK
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
	dpbsv_( &uplo, &nfreedom, &kb, &nrhs, band, &ld_band, tempx , &ld_rhs, &info);
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#else
	work=vectorR (  nfreedom );
#if LOWER
	dbandc_(band,  &ld_band, & nfreedom, work);
	dbandsc_(band, &ld_band, & nfreedom, tempx);
#else
	dbandtc_(band,  &ld_band, & nfreedom);
	dbandtsc_(band, &ld_band, & nfreedom, tempx);
#endif
	free_vectorR (work);
#endif
/** transform from tempx to temp ***/
	set_temp();
/** show displacement **/
	show_displacement( fp_out );
/*** freeing allocated memory ***/ 
	free_vectorR(temp);
	free_vectorR(tempx);
/*** show rforce ***/
	show_rforce( fp_out);
/*** show stress ***/
	show_stress( fp_out );
/*** write vtf ***/
	writeVTF( fp_log, fp_vtf, ct3d);
/*** write plot ***/
	writePLT( fp_log, fp_plt, ct3d);
/*** message **/
	fprintf( fp_out,"### solveSTATIC FINISHED!\n");
	fprintf( fp_log,"### solveSTATIC FINISHED!\n");
	fflush( fp_log );
}
/*************************
	set load
**************************/
static void set_load(FILE* fp_log)
{
	int i;
	int inod;
	double val;
	int id,dof_id;
/***** clear array ***/
	for(i=0;i<nfreedom;i++){
		tempx[i]=0.0;
	}
/*** cload   ****/
	for(i=0;i< ncload;i++){
		inod     =cload[i].node_id;
		dof_id   =cload[i].dof_id;
		val      =cload[i].value;
		id=doftab[ NDOF*(inod-1)+dof_id-1 ];
		if( id > 0 ) tempx[id-1]+=val;
	}
	fprintf(fp_log,"### set load finished!\n");
	fflush(fp_log);
}
/*************************
 assemble global matrix
**************************/
static void assemble(FILE *fp)
{
	double stf[NNOD*NDOF*(NNOD*NDOF+1)/2];
	int nid[NNOD],iwk[NNOD*NDOF];
	int i,j,k,nn;
	int head;
	double xx[NNOD],yy[NNOD],zz[NNOD];
	double young,area;
	int mid;
	int sid;
/*** truss *****/
	nn=2;
	for(i=0;i<nume;i++){
		for(j=0;j<nn;j++){
			nid[j]=truss[i].id[j];
		}
/** coordinate of 2-node **/
		for(j=0;j<nn;j++){
			xx[j]=node[nid[j]-1].cord.x;
			yy[j]=node[nid[j]-1].cord.y;
			zz[j]=node[nid[j]-1].cord.z;
		}
/** section **/
		sid=truss[i].sid;
		area=section[sid-1].area;
		mid =section[sid-1].mid;
/** material **/
		young=material[mid-1].young;	
/** head **/
		head=0;
		for(j=0;j<nn;j++){
			for(k=0;k<NDOF;k++){
				iwk[ head + k ]=doftab[NDOF*(nid[j]-1)+k];
			}
			head+=NDOF;
		}
/*** 2-node 3D Truss Element **/
		mk_t3(fp,xx,yy,zz,young,area,stf);
/***add global stiffness *****/
		add_gmat(iwk,stf,head);
	}
}
/****************************
   add global stiffness    
****************************/
static void add_gmat(int iwk[], double stf[], int nn)
{   
	int ii,jj,ix,jx,num;
	double sss;
/** loop over all dofs **/
	for(jj=0;jj<nn;jj++){
		jx=iwk[ jj ];
/* OFF DIAGONAL TERM */
		for( ii=0;ii<jj;ii++){
			num=jj*(jj+1)/2+ii;
			ix =iwk[ ii ];
			sss=stf[ num ];
			if( ix > 0 && jx > 0 ){
/* BOTH INDEPENDED */
				if( jx > ix ) {
#if LOWER
					band[ (kb+1)*(ix-1) + jx-ix ]+=sss;
#else
					band[ (kb+1)*(jx-1) + kb+ix-jx ]+=sss;
#endif
				}else{
#if LOWER
					band[ (kb+1)*(jx-1) + ix-jx ]+=sss;
#else
					band[ (kb+1)*(ix-1) + kb+jx-ix ]+=sss;
#endif
				}
			}else if( ix > 0 && jx < 0 ){
/* CONSIDERING CONSTRAINT CONDITION FOR JX */
				tempx[ix-1]+=-sss*cval[-jx-1];
			}else if( ix < 0 && jx > 0 ){
/* CONSIDERING CONSTRAINT CONDITION FOR IX */
				tempx[jx-1]+=-sss*cval[-ix-1];
			}else if( ix < 0 && jx < 0 ){
/* NOTHING TO DO */
			}
		}
/* DIAGONAL TERM */
		num=(jj+1)*(jj+2)/2-1;
		sss=stf[num];
		if( jx > 0 ){
#if LOWER
			band[ (kb+1)*(jx-1) ]+=sss;
#else
			band[ (kb+1)*(jx-1) + kb ]+=sss;
#endif
		}else if( jx < 0 ){
/* NOTHING TO DO */
		}
	}
}
/***************************
 Transform from tempx to temp
****************************/
static void set_temp( FILE* fp )
{
	int i,j,jj;
	double val;
/** Loopover nodes ***/
	for(i=0;i<numnp;i++){
		for(j=0;j<NDOF;j++){
			jj=doftab[NDOF*i+j];
			if( jj > 0 ){
				temp[NDOF*i+j]=tempx[jj-1];
			}else if( jj < 0 ){
				val=cval[-jj-1];
				temp[NDOF*i+j]=val;
			}
		}
	}
}
/*********************
	Show Displacement   
**********************/
static void show_displacement( FILE* fp)
{
	int i;
	fprintf(fp,"*** Displacement (U1/U2/U3)****\n");
	for(i=0;i<numnp;i++){
		fprintf(fp,"%5d %10.4e %10.4e %10.4e\n",i+1,
		temp[3*i],temp[3*i+1],temp[3*i+2]);
		result_node[i].u[0]=temp[3*i  ];
		result_node[i].u[1]=temp[3*i+1];
		result_node[i].u[2]=temp[3*i+2];
	}
}
/**************************************************************
	Show Stress
***************************************************************/
static void show_stress( FILE *fp)
{
	int i,j,k;
	double xx[NNOD],yy[NNOD],zz[NNOD];
	int mid;
	int sid;
	int nid[NNOD];
	double disp[NNOD*NDOF];
	int nn;
	double young,area;
	int count=0;
/** stress ***/
	double s11;
/* Header */
	fprintf(fp,"*** Stress (S11)****\n");
/*** truss *****/
	nn=2;
	for(i=0;i<nume;i++) {
		for(j=0;j<nn;j++) nid[j]=truss[i].id[j];
/** coordinate of 2-node **/
		for(j=0;j<nn;j++){
			xx[j]=node[nid[j]-1].cord.x;
			yy[j]=node[nid[j]-1].cord.y;
			zz[j]=node[nid[j]-1].cord.z;
		}
/** section **/
		sid=truss[i].sid;
		area=section[sid-1].area;
		mid =section[sid-1].mid;
/** material **/
		young=material[mid-1].young;
/** Local Displacment **/
		for(j=0;j<nn;j++){
			for(k=0;k<NDOF;k++){
				disp[NDOF*j+k]=result_node[nid[j]-1].u[k];
			}
		}
/*** recover truss element ***/
		rcv_t3(fp,xx,yy,zz,young,area,disp,&s11);
		fprintf(fp,"### Element %d : %e\n",++count, s11);
/*** set stress  ***/
		result_element[i].s11=s11;
	}
}
/***************************************
	Show reaction force
***************************************/
static void show_rforce(FILE* fp)
{
	int i,j,k,nn;
	int flag;
	int nid[NNOD];
	double *spcForce;
	int count=0;
/** Allocate ***/
	spcForce=vectorR( nspc );
/*** 2-node element ***/
	nn=2;
	for(i=0;i<nume;i++){
		for(j=0;j<nn;j++){
			nid[j]=truss[i].id[j];
		}
		flag=0;
		for(j=0;j<nspc;j++){
			for(k=0;k<nn;k++){
				if( spc[j].node_id == nid[k] ) flag=1;
			}
		}
		if( flag == 1 ) add_force(fp, count, spcForce);
		count++;
	}
	fprintf(fp,"#### Reaction Force #####\n");
	for(i=0;i<nspc;i++){
		fprintf(fp,"spc force id=%d dof=%d force=%10.4f\n",
		spc[i].node_id,
		spc[i].dof_id,
		spcForce[i]);
	}
/** Freeing ***/
	free_vectorR( spcForce );
	fflush(fp);
}
/***************************************************
     Add Nodal Force
***************************************************/
static void add_force(FILE* fp,int eid, double* spcForce)
{
	int i,j,k,ii,jj;
	double stf[NNOD*NDOF*(NNOD*NDOF+1)/2];
	int nn=2;
	int nid[NNOD];
	double xx[NNOD],yy[NNOD],zz[NNOD];
	int nsize,count;
	int mid;
	int sid;
	int list[NNOD*NDOF]; 
	double disp[NNOD*NDOF];
	double force[NNOD*NDOF];
	int num;
	double young,area;
	for( j=0;j<nn;j++){
		nid[j]=truss[eid].id[j];
	}
	
/** nodal coordinate **/
	for(j=0;j<nn;j++){
		xx[j]=node[nid[j]-1].cord.x;
		yy[j]=node[nid[j]-1].cord.y;
		zz[j]=node[nid[j]-1].cord.z;
	}
/** Section **/
	sid=truss[eid].sid;
	area=section[sid-1].area;
	mid =section[sid-1].mid;
/** Material **/
	young =material[mid-1].young;
/** Local Displacment **/
	for(j=0;j<nn;j++){
		for(k=0;k<NDOF;k++){
			disp[NDOF*j+k]=result_node[nid[j]-1].u[k];
		}
	}
/*** 2 node 3D Truss Element **/
	mk_t3(fp,xx,yy,zz,young,area,stf);
/** Size of element stiffness **/
	nsize=NDOF*nn;
/*** [K]{U} ***/
	for(jj=0;jj<nsize;jj++) force[jj]=0.0;
	for(jj=0;jj<nsize;jj++){
		for(ii=0;ii< jj;ii++){
			num=jj*(jj+1)/2 + ii;
			force[ii]+=stf[num]*disp[jj];
			force[jj]+=stf[num]*disp[ii];
		}
		num=jj*(jj+1)/2 + jj;
		force[jj]+=stf[num]*disp[jj];
	}
/*** check SPC node ***/
	for(j=0;j<nsize;j++){
		list[j]=0;
	}
	count=0;
	for(j=0;j<nn;j++){
		for(k=0;k<NDOF;k++){
			for(i=0;i<nspc;i++){
				ii=spc[i].node_id;
				jj=spc[i].dof_id;
				if( nid[j] == ii && (k+1) == jj ){
					list[count]=i+1;
				}
			}
			count++;
		}
	}
/*** Add glForce ***/
	for(jj=0;jj<nsize;jj++){
		if( list[jj] > 0 ){
			spcForce[list[jj]-1]+=force[jj];
		}
	}
}
/***************************************************
    write PLOT file
***************************************************/
static void writePLT(FILE* fp_log, FILE* fp_plt, CT3D *ct3d)
{
	int i;
/** title ***/
	fprintf(fp_plt,"%s\n",ct3d->title);
/** Number of nodes, elements ***/
	fprintf(fp_plt,"%d %d\n",ct3d->numnp,ct3d->nume);
/** nodes ***/
	for(i=0;i<ct3d->numnp;i++){
		fprintf(fp_plt,"%e %e %e\n",
		(ct3d->node)[i].cord.x,(ct3d->node)[i].cord.y,(ct3d->node)[i].cord.z);
	}
/** elements ***/
	for(i=0;i<ct3d->nume;i++){
		fprintf(fp_plt,"%d %d\n",
		(ct3d->truss)[i].id[0],
		(ct3d->truss)[i].id[1]);
	}
/** Results (displacement) ***/
	for(i=0;i<ct3d->numnp;i++){
		fprintf(fp_plt,"%e %e %e\n",
		(ct3d->result_node)[i].u[0],
		(ct3d->result_node)[i].u[1],
		(ct3d->result_node)[i].u[2]);
	}
/** Results (stress) ***/
	for(i=0;i<ct3d->nume;i++){
		fprintf(fp_plt,"%e\n",(ct3d->result_element)[i].s11);
	}
	fflush( fp_plt );
}
