!************************************************************************
!      basic procedure for FT2D solver                                  *
!************************************************************************
!*************************
!    create dofs table
!*************************/
!-----------------------------------------------------------------------!
      subroutine create_doftab(ft2d)
!-----------------------------------------------------------------------!
      use ft2d_m
      implicit none
!
! INTERFACE
!
      type(strFT2D) :: ft2d
!
! LOCAL
!
      integer (kind=4) :: i,j,ii,jj,kk
      integer (kind=4) :: ncon,nfreedom
! Allocate
      allocate( ft2d%doftab( NDOF * ft2d%numnp ) )
! set fill 1 
      do i=1,ft2d%numnp
        do j=1,NDOF
          ft2d%doftab( NDOF*(i-1)+j )=1
        enddo
      enddo
! set spc 
      ncon=0
      do i=1,ft2d%nspc
        ii =ft2d%spc(i)%node_id
        jj =ft2d%spc(i)%dof_id
        kk =NDOF*(ii-1)+jj
        ncon=ncon+1
        ft2d%doftab(kk)=-ncon
      enddo
! alive node
      nfreedom=0
      do i=1, NDOF*ft2d%numnp
        if( ft2d%doftab(i) .eq. 1 ) then
          nfreedom=nfreedom+1
          ft2d%doftab(i)=nfreedom  
        endif
      enddo
! show table
      if( DEBUG .eq. 1 ) then
	write(ILOG,*) "### DOFTAB List ####"
        do i=1,ft2d%numnp
          write(ILOG,'(''#node id='',i5,10i8)') i+1,(ft2d%doftab(NDOF*(i-1)+j),j=1,NDOF) 
        enddo
      endif
      write(IOUT,*) "### number of freedoms=",nfreedom
      write(IOUT,*) "### number of constrain=",ncon
      ft2d%nfreedom=nfreedom
      ft2d%ncon=ncon
! message
      write(ILOG,*) "### create_doftab FINISHED!"
      end subroutine create_doftab
!**************************************
!	Create_ConstList
!**************************************
!-----------------------------------------------------------------------!
      subroutine create_constList(ft2d)
!-----------------------------------------------------------------------!
      use ft2d_m
      implicit none
!
! INTERFACE
!
      type(strFT2D) :: ft2d
!
! LOCAL
!
      integer (kind=4) :: i,ii,id,id1
      real    (kind=8) :: value
!**************************
!  Allocation
!**************************
      if( ft2d%ncon .gt. 0 ) allocate( ft2d%cval( ft2d%ncon ) )
!**************************
! Spc Loop
!**************************
      do i=1, ft2d%nspc
        id=ft2d%spc(i)%node_id
        id1=ft2d%spc(i)%dof_id
        value=ft2d%spc(i)%value
        if( ft2d%doftab( NDOF*(id-1)+id1 ) .lt. 0 ) then
          ii=-ft2d%doftab( NDOF*(id-1)+id1)
          ft2d%cval(ii)=value
        endif
      enddo
!**************************
! Show
!**************************
      write(IOUT,*) "#### Number of nodes            =",ft2d%numnp
      write(IOUT,*) "#### Number of freedoms         =",ft2d%nfreedom
      write(IOUT,*) "#### Number of Constrained DOFs =",ft2d%ncon
      write(ILOG,*) "#### Number of nodes            =",ft2d%numnp
      write(ILOG,*) "#### Number of freedoms         =",ft2d%nfreedom
      write(ILOG,*) "#### Number of Constrained DOFs =",ft2d%ncon
! message
      write(ILOG,*) "### create_constList FINISHED!"
      end subroutine create_constList
!*************************
! find size of band matrix
!*************************
!-----------------------------------------------------------------------!
      integer function find_bw(ft2d)
!-----------------------------------------------------------------------!
      use ft2d_m
      implicit none
!
! INTERFACE
!
      type(strFT2D) :: ft2d
!
! LOCAL
!
      integer (kind=4) :: i,nn,ls,j,k,me
      integer (kind=4) :: nid(NNOD),nd(NNOD*NDOF)
      integer (kind=4) :: mht
      integer (kind=4) :: head
! clear height
      mht=0
! truss element
      nn=2
      do i=1,ft2d%nume
        do j=1,nn
          nid(j)=ft2d%truss(i)%id(j)
        enddo
! Set dofs  per node
        head=0
        do j=1,nn
          do k=1,NDOF
            nd(head+k)=ft2d%doftab(NDOF*(nid(j)-1)+k)
          enddo
          head=head+NDOF
        enddo
	ls=ft2d%numnp*NDOF
        do j=1,head
          if( nd(j) .gt. 0 ) then
            if( nd(j) .lt. ls ) ls=nd(j)
          endif
        enddo

        do j=1,head
          if( nd(j) .gt. 0 ) then
            me=nd(j)-ls+1
            if( me .gt. mht ) mht=me
          endif
        enddo
      enddo
      write(IOUT,*) "*** B A N D    M A T R I X   I N F O R M A T I O N ****"
      write(IOUT,*) "#### Number of subdiagonals",mht-1
      write(ILOG,*) "*** B A N D    M A T R I X   I N F O R M A T I O N ****"
      write(ILOG,*) "#### Number of subdiagonals",mht-1
! message 
      write(ILOG,*) "### find_bw FINISHED!"
      find_bw=mht-1
      end function find_bw
