!************************************************************************
!         data io module for FT2D  solver                                    
!************************************************************************
!********************
! read input data     
!********************
!-----------------------------------------------------------------------!
      subroutine read_data(ft2d)
!-----------------------------------------------------------------------!
      use ft2d_m
      implicit none
!
! INTERFACE
!
      type(strFT2D) :: ft2d
!
! LOCAL
!
      integer (kind=4) :: i
      type(strCORD) :: cord
!**********************
!* read control data
!**********************
      read(IIN,*) ft2d%title
! General
      read(IIN,*) ft2d%numnp
      read(IIN,*) ft2d%nume
      read(IIN,*) ft2d%numat
      read(IIN,*) ft2d%nsect
      read(IIN,*) ft2d%nspc
      read(IIN,*) ft2d%ncload
!*********************
! read node data
!*********************
      allocate( ft2d%node(ft2d%numnp ) )
      do i=1,ft2d%numnp
        read(IIN,*) cord%x,cord%y
        ft2d%node(i)%cord=cord
      enddo
!**********************
! read element data 
!**********************
! truss
      allocate( ft2d%truss(ft2d%nume ) )
      do i=1,ft2d%nume
        read(IIN,*) ft2d%truss(i)%sid,ft2d%truss(i)%id(1),ft2d%truss(i)%id(2)
      enddo
!**********************
! read material data
!**********************
      allocate( ft2d%material(ft2d%numat) )
      do i=1,ft2d%numat
        read(IIN,*) ft2d%material(i)%young
      enddo
!**********************
! read section data 
!**********************
      allocate( ft2d%section(ft2d%nsect ) )
      do i=1,ft2d%nsect
        read(IIN,*) ft2d%section(i)%area,ft2d%section(i)%mid
      enddo
!********************
! read spc data  
!********************
      if( ft2d%nspc .gt. 0 ) then
        allocate( ft2d%spc(ft2d%nspc ) )
        do i=1,ft2d%nspc
          read(IIN,*) ft2d%spc(i)%node_id,ft2d%spc(i)%dof_id,ft2d%spc(i)%value
        enddo
      endif
!******************
! read cload data
!******************
      if( ft2d%ncload .gt. 0 ) then
        allocate( ft2d%cload(ft2d%ncload) )
        do i=1,ft2d%ncload
          read(IIN,*) ft2d%cload(i)%node_id,ft2d%cload(i)%dof_id,ft2d%cload(i)%value 
        enddo
      endif
! message 
      write(ILOG,*) "### read_data FINISHED!"
      end subroutine read_data
!***********************
!  input data echo
!***********************
!-----------------------------------------------------------------------!
      subroutine input_echo(ft2d)
!-----------------------------------------------------------------------!
      use ft2d_m
      implicit none
!
! INTERFACE
!
      type(strFT2D) :: ft2d
!
! LOCAL
!
      type(strCORD) :: cord
      integer (kind=4) :: i,nid
      write(IOUT,*) "///////////////////////////////////////////////"
      write(IOUT,*) "// FT2D Solver Ver.1.1 (Band Matrix Version) //"
      write(IOUT,*) "///////////////////////////////////////////////"
      write(IOUT,*) "VERSION",VERSION
      if( LAPACK .eq. 1 ) then
        write(IOUT,*) "Using LAPACK"
      else 
        write(IOUT,*) "Using SAMUKAWA Solver"
      endif
      if( LOWER .eq. 1 ) then
        write(IOUT,*) "Storing Lower Triangle Matrix in Band form"
      else 
        write(IOUT,*) "Storing Upper Triangle Matrix in Band form"
      endif
      write(IOUT,*) "**** control data *****"
      write(IOUT,*) " Title:",trim(ft2d%title)
      write(IOUT,*) " number of nodes          (numnp    )",ft2d%numnp
      write(IOUT,*) " number of truss elements (nume     )",ft2d%nume
      write(IOUT,*) " number of materials      (numat    )",ft2d%numat
      write(IOUT,*) " number of sections       (nsect    )",ft2d%nsect
      write(IOUT,*) " number of spc            (nspc     )",ft2d%nspc
      write(IOUT,*) " number of cload          (ncload   )",ft2d%ncload
      write(IOUT,*) "**** node data *****"
      do i=1,ft2d%numnp
	  cord=ft2d%node(i)%cord
	  write(IOUT,*) i,":",cord%x, cord%y
      enddo
      write(IOUT,*) "**** element data *****"
      write(IOUT,*) "**** truss element ****"
      do i=1,ft2d%nume
        write(IOUT,*) i,":secid=", ft2d%truss(i)%sid,ft2d%truss(i)%id(1),ft2d%truss(i)%id(2)
      enddo
      write(IOUT,*) "***** material properties"
      do i=1,ft2d%numat
        write(IOUT,*) "matid    =",i
        write(IOUT,*) "young    =",ft2d%material(i)%young
      enddo 
      write(IOUT,*) "***** section properties"
      do i=1,ft2d%nsect
        write(IOUT,*) "secid    =",i+1
        write(IOUT,*) "area     =",ft2d%section(i)%area
        write(IOUT,*) "matid    =",ft2d%section(i)%mid
      enddo
      if( ft2d%nspc .gt. 0) then
        write(IOUT,*) "**** spc ****"
        do i=1,ft2d%nspc
          nid=ft2d%spc(i)%node_id
          write(IOUT,*) "SPC(",i,"):nid=",ft2d%spc(i)%node_id,"dof=",ft2d%spc(i)%dof_id,"value=",ft2d%spc(i)%value
        enddo
      endif
      if( ft2d%ncload .gt. 0) then
        write(IOUT,*) "**** cload data  ****"
        do i=1,ft2d%ncload
          nid=ft2d%cload(i)%node_id
          write(IOUT,*) "CLOAD(",i,"):nid=",ft2d%cload(i)%node_id,"dof=",ft2d%cload(i)%dof_id,"value=",ft2d%cload(i)%value
        enddo
      endif
! message
      write(ILOG,*) "### input_echo FINISHED!"
      end subroutine input_echo
