!******************************************************
!  Solver STATIC MODULE
!******************************************************
      module solveSTATIC_m
      use ft2d_m
      implicit none 
!
      private
!
! variables
!
      character (len=NAME_LENG) ,save:: title
      integer (kind=4) ,save :: numnp
      integer (kind=4) ,save :: nume
      integer (kind=4) ,save :: numat
      integer (kind=4) ,save :: nsect
      integer (kind=4) ,save :: nspc
      integer (kind=4) ,save :: ncload
      type    (strNODE)    ,pointer ,save :: node(:)
      type    (strTRUSS)   ,pointer ,save :: truss(:)
      type    (strMATERIAL),pointer ,save :: material(:)
      type    (strSECTION ),pointer ,save :: section(:)
      type    (strSPC)     ,pointer ,save :: spc(:)
      type    (strCLOAD)   ,pointer ,save :: cload(:)
      integer (kind=4) ,save :: nfreedom
      integer (kind=4) ,save :: ncon
      integer (kind=4) ,pointer ,save :: doftab(:)
      real    (kind=8) ,pointer ,save :: tempx (:)
      real    (kind=8) ,pointer ,save :: temp  (:)
      real    (kind=8) ,pointer ,save :: cval  (:)
      type(strRESULT_NODE)   ,pointer, save :: result_node(:)
      type(strRESULT_ELEMENT),pointer, save :: result_element(:)
!
! public procedure
!
      public :: solveSTATIC
!
      contains
!******************************
!  static solver
!******************************
!-----------------------------------------------------------------------!
      subroutine solveSTATIC(ft2d)
!-----------------------------------------------------------------------!
      use ft2d_m
      use matrix_m
      implicit none
!
! INTERFACE
!
      type(strFT2D)    :: ft2d
!
! LOCAL
!
      integer (kind=4) :: i,j,ii,jj,kk
      real    (kind=8), pointer:: work(:)
      integer (kind=4) ::find_bw
! Start 
      write(IOUT,*) "######################" 
      write(IOUT,*) "#### Static Analysis" 
      write(IOUT,*) "######################" 
! Allocate array
      allocate ( ft2d%temp( NDOF * ft2d%numnp ) )
      allocate ( ft2d%tempx( ft2d%nfreedom ) )
      allocate ( ft2d%result_node( ft2d%numnp ) )
      allocate ( ft2d%result_element( ft2d%nume ) )
!
! SET VARIABLES
!
      title   =ft2d%title
      numnp   =ft2d%numnp
      nume    =ft2d%nume
      numat   =ft2d%numat
      nsect   =ft2d%nsect
      nspc    =ft2d%nspc
      ncload  =ft2d%ncload
      node     => ft2d%node
      truss    => ft2d%truss
      material => ft2d%material
      section  => ft2d%section
      spc      => ft2d%spc
      cload    => ft2d%cload
      nfreedom =  ft2d%nfreedom
      ncon     =  ft2d%ncon
      doftab   => ft2d%doftab
      tempx    => ft2d%tempx
      temp     => ft2d%temp
      cval     => ft2d%cval
      result_node   => ft2d%result_node
      result_element=> ft2d%result_element
! size of band matrix
      kb=find_bw(ft2d)
! allocate bandmatrix 
      allocate( band( (kb+1)* ft2d%nfreedom ) )
! create load vector ***  
      call set_load
! create & assemble stiffness matrix 
      call assemble
! Solve Band Matrix
      ld_band=kb+1
      ld_rhs=ft2d%nfreedom
! Select solver 
      if( LAPACK.eq.1 ) then
        if( LOWER.eq.1 ) then
          uplo='L'
        else
          uplo='U'
        endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!       call dpbsv(uplo,ft2d%nfreedom,kb,nrhs,band,ld_band,ft2d%tempx,ld_rhs,info) 
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
      else
        allocate( work(ft2d%nfreedom) )
        if( LOWER.eq.1 ) then
        else 
          call dbandt  (band,ld_band, ft2d%nfreedom)
          call dbandts (band,ld_band, ft2d%nfreedom, ft2d%tempx)
        endif
        deallocate( work )
      endif
! transform from tempx to temp
      call set_temp
! show displacement
      call show_displacement
! freeing allocated memory
      deallocate( ft2d%temp )
      deallocate( ft2d%tempx)
! show rforce
      call show_rforce
! show stress
      call show_stress
! write vtf
      call writeVTF(ft2d)
! write plot
      call writePLT
! message
      write(IOUT,*) "### solveSTATIC FINISHED!"
      write(ILOG,*) "### solveSTATIC FINISHED!"
      end subroutine solveSTATIC
!************************
!     set load
!*************************
!-----------------------------------------------------------------------!
      subroutine set_load
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      integer (kind=4) :: i,inod,id,dof_id
      real    (kind=8) :: val
!**** clear array
      do i=1,nfreedom
        tempx(i)=0.0
      enddo
!** cload   ****/
      do i=1,ncload
        inod     =cload(i)%node_id
        dof_id   =cload(i)%dof_id
        val      =cload(i)%value
        id=doftab( NDOF*(inod-1) + dof_id)
        if( id .gt. 0 ) tempx(id)=tempx(id)+val
      enddo
      write(ILOG,*) "### set load finished!"
      end subroutine set_load
!************************
!  assemble global matrix
!************************
!-----------------------------------------------------------------------!
      subroutine assemble
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      real    (kind=8) :: stf(NNOD*NDOF*(NNOD*NDOF+1)/2)
      integer (kind=4) :: nid(NNOD),iwk(NNOD*NDOF)
      integer (kind=4) :: i,j,k
      integer (kind=4), parameter :: nn=2
      integer (kind=4) :: head
      real    (kind=8) :: xx(NNOD),yy(NNOD)
      real    (kind=8) :: young,area
      integer (kind=4) :: mid,sid
! truss 
      do i=1,nume
        do j=1,nn
          nid(j)=truss(i)%id(j)
        enddo
! coordinate of 2-node
        do j=1,nn
          xx(j)=node( nid(j) )%cord%x
          yy(j)=node( nid(j) )%cord%y
        enddo
! section
        sid =truss(i)%sid
        area=section(sid)%area
        mid =section(sid)%mid
! material
        young=material(mid)%young	
! head 
        head=0
        do j=1,nn
          do k=1,NDOF
            iwk( head + k )=doftab(NDOF*(nid(j)-1)+ k)
          enddo
          head=head+NDOF
        enddo
! 2-node Element
        call mk_t2(ILOG,xx,yy,young,area,stf)
! add global stiffness 
        call add_gmat(iwk,stf,head)
      enddo
      write(ILOG,*) "### assemble finished!"
      end subroutine assemble
!***************************
!   add global stiffness    
!***************************
!-----------------------------------------------------------------------!
      subroutine add_gmat(iwk, stf, nn)
!-----------------------------------------------------------------------!
      use matrix_m
      implicit none
!
! INTERFACE
!
      integer (kind=4) :: iwk(NNOD*NDOF)
      real    (kind=8) :: stf(NNOD*NDOF*(NNOD*NDOF+1)/2)
      integer (kind=4) :: nn
!
! LOCAL
!
      integer (kind=4) :: ii,jj,ix,jx,num
      real    (kind=8) :: sss
! loop over all dofs
      do jj=1,nn
        jx=iwk(jj)
! OFF DIAGONAL TERM
        do ii=1,jj-1
          num=(jj-1)*jj/2+ii
          ix=iwk(ii)
          sss=stf(num)
          if( ix.gt.0 .and. jx.gt.0 ) then
! BOTH INDEPENDED
            if( jx .gt. ix ) then
!-----------------------------------------------------------------------!
              if(LOWER.eq.1) then
                band((kb+1)*(ix-1)+jx-ix+1)=band((kb+1)*(ix-1)+jx-ix+1)+sss
              else
                band((kb+1)*(jx-1)+kb+ix-jx+1)=band((kb+1)*(jx-1)+kb+ix-jx+1)+sss
              endif
            else 
              if(LOWER.eq.1) then
                band((kb+1)*(jx-1)+ix-jx+1)=band((kb+1)*(jx-1)+ix-jx+1)+sss   
              else
                band((kb+1)*(ix-1)+kb+jx-ix+1)=band((kb+1)*(ix-1)+kb+jx-ix+1)+sss
              endif
            endif
          else if( ix.gt.0 .and. jx.lt.0 ) then
! CONSIDERING CONSTRAINT CONDITION FOR JX
            tempx(ix)=tempx(ix)-sss*cval(-jx)
          else if( ix.lt.0 .and. jx.gt.0 ) then
! CONSIDERING CONSTRAINT CONDITION FOR IX
            tempx(jx)=tempx(jx)-sss*cval(-ix)
          else if( ix.lt.0 .and. jx.lt.0 ) then
! NOTHING TO DO
          endif
        enddo
! DIAGONAL TERM
        num=(jj-1)*jj/2+jj
        sss=stf(num)
        if( jx.gt.0 ) then
          if(LOWER.eq.1) then
            band((kb+1)*(jx-1)+1)=band((kb+1)*(jx-1)+1)+sss
          else
            band((kb+1)*(jx-1)+kb+1)=band((kb+1)*(jx-1)+kb+1)+sss
          endif
        else if( jx.lt.0 ) then
! NOTHING TO DO
        endif
      enddo
      end subroutine add_gmat
!**************************
! Transform from tempx to temp
!***************************
!-----------------------------------------------------------------------!
      subroutine set_temp
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      integer (kind=4) :: i,j,jj
      real    (kind=8) :: val
! Loopover nodes
      do i=1,numnp
        do j=1,NDOF
          jj=doftab(NDOF*(i-1)+j)
          if( jj.gt.0 ) then
            temp(NDOF*(i-1)+j)=tempx(jj)
          else if( jj.lt.0 ) then
            temp(NDOF*(i-1)+j)=cval(-jj)
          endif
        enddo
      enddo
      end subroutine set_temp
!**!******************
!   Show Displacement   
!*********************
!-----------------------------------------------------------------------!
      subroutine show_displacement
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      integer (kind=4) :: i
! show
      write(IOUT,*) "*** Displacement (U1/U2)****"
      do i=1,numnp
        write(IOUT,'(I5,2E15.7)') i,temp(2*i-1),temp(2*i)
        result_node(i)%u(1)=temp(2*i-1)
        result_node(i)%u(2)=temp(2*i)
      enddo
      end subroutine show_displacement
!*************************************************************
!     Show Stress
!*************************************************************
!-----------------------------------------------------------------------!
      subroutine show_stress
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      integer (kind=4) :: i,j,k
      real    (kind=8) :: xx(NNOD),yy(NNOD)
      integer (kind=4) :: mid,sid,nid(NNOD)
      real    (kind=8) :: disp(NNOD*NDOF)
      integer (kind=4) , parameter :: nn=2
      real    (kind=8) :: young,area
      integer (kind=4) :: count=0
! stress
      real    (kind=8) :: s11
! Header
      write(IOUT,*) "*** Stress (S11)****"
! truss
      do i=1,nume
        do j=1,nn
          nid(j)=truss(i)%id(j)
        enddo
! coordinate of 2-node
        do j=1,nn
          xx(j)=node(nid(j))%cord%x
          yy(j)=node(nid(j))%cord%y
        enddo
! section
        sid =truss(i)%sid
        area=section(sid)%area
        mid =section(sid)%mid
! material
        young=material(mid)%young
! Local Displacment
        do j=1,nn
          do k=1,NDOF
            disp(NDOF*(j-1)+k)=result_node(nid(j))%u(k)
          enddo
        enddo
! recover truss element
        call rcv_t2(IOUT,xx,yy,young,area,disp,s11)
        count=count+1
        write(IOUT,'(''### Element'',I5 ,'':'',E15.7)') count,s11
! set stress
        result_element(i)%s11=s11
      enddo
      end subroutine show_stress
!**************************************
!     Show reaction force
!**************************************
!-----------------------------------------------------------------------!
      subroutine show_rforce
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      integer (kind=4) :: i,j,k,flag,nid(NNOD)
      real(kind=8),pointer :: spcForce(:)
      integer (kind=4) :: count=0
      integer (kind=4) , parameter :: nn=2
! Allocate
      allocate( spcForce(nspc ) )
! 2-node element
      do i=1,nume
        do j=1,nn
          nid(j)=truss(i)%id(j)
        enddo
        flag=0
        do j=1,nspc
          do k=1,nn
            if( spc(j)%node_id .eq. nid(k) ) flag=1
          enddo
        enddo
        count=count+1
        if( flag.eq.1 ) then
            call add_force(count,spcForce)
        endif
      enddo
      write(IOUT,*) "#### Reaction Force #####"
      do i=1,nspc
        write(IOUT,'(''spc force id='',I5,''dof='',I5,''force='',E15.7)') spc(i)%node_id,spc(i)%dof_id,spcForce(i)       
      enddo
! Freeing
      deallocate(spcForce)
      end subroutine show_rforce
!**************************************************
!     Add Nodal Force
!**************************************************
!-----------------------------------------------------------------------!
      subroutine add_force(eid,spcForce)
!-----------------------------------------------------------------------!
      implicit none
!
! INTERFACE
!
      integer (kind=4) :: eid
      real    (kind=8) :: spcForce(:)
!
! LOCAL
!
      integer (kind=4) :: i,j,k,ii,jj
      real    (kind=8) :: stf(NNOD*NDOF*(NNOD*NDOF+1)/2)
      integer (kind=4) :: nn=2
      integer (kind=4) :: nid(NNOD)
      real    (kind=8) :: xx(NNOD),yy(NNOD)
      integer (kind=4) :: nsize,count
      integer (kind=4) :: mid,sid,list(NNOD*NDOF)
      real    (kind=8) :: disp(4),force(NNOD*NDOF)
      integer (kind=4) :: num
      real    (kind=8) :: young,area
      do j=1,nn
        nid(j)=truss(eid)%id(j)
      enddo	
! nodal coordinate
      do j=1,nn
        xx(j)=node( nid(j) )%cord%x
        yy(j)=node( nid(j) )%cord%y
      enddo
! Section
      sid =truss(eid)%sid
      area=section(sid)%area
      mid =section(sid)%mid
! Material
      young=material(mid)%young
! Local Displacment
      do j=1,nn
        do k=1,NDOF
          disp( NDOF*(j-1)+k )=result_node(nid(j))%u(k)
        enddo
      enddo
! 2 node Element
      call mk_t2(IOUT,xx,yy,young,area,stf)
! Size of element stiffness
      nsize=NDOF*nn
! [K]{U}
      do jj=1,nsize
        force(jj)=0.0
      enddo
      do jj=1,nsize
        do ii=1,jj-1
          num=(jj-1)*jj/2+ii
          force(ii)=force(ii)+stf(num)*disp(jj)
          force(jj)=force(jj)+stf(num)*disp(ii)
        enddo
        num=(jj-1)*jj/2+jj
        force(jj)=force(jj)+stf(num)*disp(jj)
      enddo
! check SPC node
      do j=1,nsize
        list(j)=0
      enddo
      count=0
      do j=1,nn
        do k=1,NDOF
          count=count+1
          do i=1,nspc
            ii=spc(i)%node_id
            jj=spc(i)%dof_id
            if( nid(j).eq.ii .and. k.eq.jj ) then
              list(count)=i
            endif
          enddo
        enddo
      enddo
! Add glForce
      do jj=1,nsize
        if( list(jj) .gt. 0 ) then
          spcForce(list(jj))=spcForce(list(jj))+force(jj)
        endif
      enddo
      end subroutine add_force
!**************************************************
! write PLOT file
!**************************************************
!-----------------------------------------------------------------------!
      subroutine writePLT
!-----------------------------------------------------------------------!
      implicit none
!
! LOCAL
!
      integer (kind=4) :: i
! title
      write(IPLT,*) title
! Number of nodes, elements
      write(IPLT,*) numnp,nume
! nodes
      do i=1,numnp
        write(IPLT,*) node(i)%cord%x,node(i)%cord%y
      enddo
! elements
      do i=1,nume
        write(IPLT,*) truss(i)%id(1),truss(i)%id(2)
      enddo
! Results (displacement)
      do i=1,numnp
        write(IPLT,*) result_node(i)%u(1),result_node(i)%u(2)
      enddo
! Results (stress)
      do i=1,nume
        write(IPLT,*) result_element(i)%s11
      enddo
      end subroutine writePLT
      end module solveSTATIC_m
