      subroutine icd (icp,irow,a,n)
!
!     incomplete Cholesky factorization
!
      implicit double precision (a-h,o-z)
      dimension icp(*),irow(*),a(*)
!
       do j=1,n
        nt=icp(j+1)-icp(j)
        j1=icp(j)-1
!                            the top-most element
         if(nt.gt.1) then
          i=irow(j1+1)
          a(j1+1)=a(j1+1)/a(icp(i+1)-1)
         endif
!                             x  x
!       sweep effects of      d  X
!       preceding columns        d
!
         if(nt.gt.2) then
          do jt=1,nt-2
           last=irow(j1+jt)
           i=irow(j1+jt+1)
           s=0.d0
            if(icp(i+1)-icp(i).gt.1) then
             s=spddot (irow(icp(i)),a(icp(i))  &
                      ,irow(icp(j)),a(icp(j)),last)
            endif
           a(j1+jt+1)=(a(j1+jt+1)-s)/a(icp(i+1)-1)
          enddo
         endif
!                                       d   X
!       now column j has changed         d  X
!                                         d X
!                                          dX
!                                           D
        s=0.d0
         if(nt.gt.1) then
           do jt=1,nt-1
            s=s+a(j1+jt)**2
           enddo
         endif
        a(j1+nt)=a(j1+nt)-s
         if(a(j1+nt).le.0.d0) then
          write(*,*) 'icd non-positive ',j,a(j1+nt),s
          stop
         endif
        a(j1+nt)=sqrt(a(j1+nt))
       enddo
      return
      end
      function spddot (irowa,a,irowb,b,last)
!
!     sparse vector ddot function
!
      implicit double precision (a-h,o-z)
      dimension irowa(*),irowb(*),a(*),b(*)
!
      s=0.d0
      i=1
      j=1
      do   
       if(irowa(i).gt.last) exit
        if(irowa(i).eq.irowb(j)) then
         s=s+a(i)*b(j)
         i=i+1
         if(irowb(j).eq.last) exit
         j=j+1
        else if(irowa(i).gt.irowb(j)) then
         if(irowb(j).eq.last) exit
         j=j+1
        else
         i=i+1
        endif
      enddo
!
      spddot=s
      return
      end
      subroutine fsolve (x,n,icp,irow,a)
      implicit double precision (a-h,o-z)
      dimension x(n),icp(n),irow(*),a(*)
!
!                   forward pass
       do j=1,n
        nt=icp(j+1)-icp(j)
        k=icp(j)
        t=x(j)
         do it=1,nt-1
          i=irow(k)
          t=t-a(k)*x(i)
          k=k+1
         enddo
        x(j)=t/a(k)
       enddo
      return
      end
      subroutine bsolve (x,n,icp,irow,a)
      implicit double precision (a-h,o-z)
      dimension x(n),icp(n),irow(*),a(*)
!
!                   backward pass
       do j=n,1,-1
        nt=icp(j+1)-icp(j)
        k=icp(j)
        x(j)=x(j)/a(k+nt-1)
        t=x(j)
         do it=1,nt-1
          i=irow(k)
          x(i)=x(i)-a(k)*t
          k=k+1
         enddo
       enddo
!
      return
      end
      subroutine dopcg (n,b,x,w,ldw,iter,resid,info &
             ,icpa,irowa,a,icppa,irowpa,pa,pq,beta,rr)
      implicit double precision (a-h,o-z)
      integer r,z,p,q
      dimension x(*),b(*),w(ldw,*),pq(*),beta(*),rr(*)
!     upper part of matrix A is stored in CCS format
!     upper part of preconditioner for matrix A is stored in CCS format
      dimension icpa(*),irowa(*),a(*),icppa(*),irowpa(*),pa(*)
!
!
!     write(*,*) 'dopcg N=',n
!     write(*,*) 'dopcg Initial B'
!     write(*,9) (b(i),i=1,n)
!
!coldriver
      dlogten=dlog(10.d0)
!
      info=0
!
!     Test the input parameters.
!
      if ( n.lt.0 ) then
         info = -1
      else if ( ldw.lt.max( 1, n ) ) then
         info = -2
      else if ( iter.le.0 ) then
         info = -3
      endif
      if ( info.ne.0 ) return
!
!     solve for transform  C^t b_hat = b
!
      call fsolve (b,n,icppa,irowpa,pa)
!
      maxit = iter
      tol   = resid
!
!     alias workspace columns.
!
      r = 1
      z = 2
      p = 3
      q = 4
!
!     set initial residual.
!
      call dcopy( n, b, 1, w(1,r), 1 )
      if ( dnrm2( n, x, 1 ).ne.zero ) then
         write(*,*) 'dopcg warning dnrm2(x) is not zero'
         call matvect(-1.d0,x,1.d0,w(1,r),n,icpa,irowa,a)
         if ( dnrm2( n, w(1,r), 1 ).lt.tol ) go to 10
      endif
      bnrm2 = dnrm2( n, b, 1 )
      rr(1)=bnrm2
      if ( bnrm2.eq.zero ) bnrm2 = one
!
!        perform preconditioned conjugate gradient iteration.
!
       do k=1,maxit
!        write(*,*)  'iter =',k
!
         rho=rr(k)**2
!
!        compute direction vector p.
!
         if ( k.gt.1 ) then
            beta(k) = rho / rho1
            call dcopy( n, w(1,r), 1, w(1,z), 1 )
            call daxpy( n, beta(k), w(1,p), 1, w(1,z), 1 )
            call dcopy( n, w(1,z), 1, w(1,p), 1 )
         else
            beta(1)=0.d0
            call dcopy( n, w(1,r), 1, w(1,p), 1 )
         endif
!
!        compute scalar alpha (save a*p to q).
!         q = C^{-1} A C^{-T} p
!
         call dcopy  (n,w(1,p),1,w(1,z),1)
         call bsolve (w(1,z),n,icppa,irowpa,pa)
         call matvect(1.d0,w(1,z),0.d0,w(1,q),n,icpa,irowa,a)
         call fsolve (w(1,q),n,icppa,irowpa,pa)
!
         pq(k)=ddot(n,w(1,p),1,w(1,q),1 )
         alpha=rho/pq(k)
!
!        compute current solution vector x.
!
         call daxpy( n, alpha, w(1,p), 1, x, 1 )
!        write(*,'(10e8.1)') (x(i),i=1,n)
!
!        compute residual vector r, find norm,
!        then check for tolerance.
!
         call daxpy( n, -alpha,  w(1,q), 1, w(1,r), 1 )
         rr(k+1)=dnrm2( n, w(1,r), 1 )
         resid = rr(k+1) / bnrm2
!
!        write(*,*) k,dlog(resid)/dlogten,resid,'  iter'
!        write(*,*) k,resid
         write(*,*) k,dlog(resid)/dlogten
!
         if ( resid.le.tol ) go to 10
!
         rho1 = rho
!
       enddo
!
!     iteration fails.
!
      info  = k
      return
!
   10 continue
!
!     Iteration successful
!
      iter=k
!
!     solve for transformation to original problem
!
      call bsolve (x,n,icppa,irowpa,pa)
!
      return
    9 format(10e8.1)
      end
