      implicit real*8 (a-h,o-z)
      complex*16 a(:),b(:),w(:)
      allocatable :: a,b,w
      real*4 tarray(2)
!
      write(6,*) 'n ='
      read(5,*) n
!
      ip=ilog2(n)
      n1=2**(ip/2)
      n2=n/n1
!
      allocate(a(n),b(n),w(n))
!
      do i=1,n
        a(i)=dcmplx(dble(i),dble(n-i+1))
        b(i)=(0.0d0,0.0d0)
      end do
!
      call settbl(w,n)
      time0=etime(tarray)
      call stockham(a,b,w,n)
      time1=etime(tarray)-time0
      flops=5.0d0*dble(ip)*dble(n)
      perf=flops/time1*1.0d-9
      write(6,*) 'Stockham FFT ',time1,'sec ',perf,' Gflops/s'
!
      do i=1,n
        a(i)=dcmplx(dble(i),dble(n-i+1))
        b(i)=(0.0d0,0.0d0)
      end do
!
      call settbl2(w,n1,n2)
      time0=etime(tarray)
      call sixstep(a,b,w,n1,n2)
      time1=etime(tarray)-time0
      flops=5.0d0*dble(ip)*dble(n)
      perf=flops/time1*1.0d-9
      write(6,*) 'block six-step FFT ',time1,'sec ',perf,' Gflops/s'
!
      deallocate(a,b,w)
      stop
      end
      subroutine stockham(a,b,w,n)
      implicit real*8 (a-h,o-z)
      complex*16 a(n),b(n),w(n)
!
      ip=ilog2(n)
!
      key=1
      j=1
      l=n
      m=1
      do i=1,ip
        l=l/2
        if (l .ge. 2) then
          if (key .ge. 0) then
            call fft2(a,b,w(j),m,l)
          else
            call fft2(b,a,w(j),m,l)
          end if
          key=-key
        else
          if (key .ge. 0) then
            call fft2(a,a,w(j),m,l)
          else
            call fft2(b,a,w(j),m,l)
          end if
        end if
        m=m*2
        j=j+l
      end do
      return
      end
      subroutine fft2(a,b,w,m,l)
      implicit real*8 (a-h,o-z)
      complex*16 a(m,l,2),b(m,2,l),w(l)
      complex*16 c0,c1
!
      do j=1,l
        do i=1,m
          c0=a(i,j,1)
          c1=a(i,j,2)
          b(i,1,j)=c0+c1
          b(i,2,j)=w(j)*(c0-c1)
        end do
      end do
      return
      end
      subroutine sixstep(x,y,w,n1,n2)
      implicit real*8 (a-h,o-z)
      parameter (nb=16,np=8)
      complex*16 x(n1,n2),y(n2,n1),w(n1,n2)
      complex*16 w1(:),w2(:),work1(:,:),work2(:)
      allocatable :: w1,w2,work1,work2
!
      allocate(w1(n1),w2(n2),work1(n2+np,nb),work2(max(n1,n2)))
!
      call settbl(w1,n1)
      call settbl(w2,n2)
!
      do ii=1,n1,nb
        do jj=1,n2,nb
          do i=ii,min(ii+nb-1,n1)
            do j=jj,min(jj+nb-1,n2)
              work1(j,i-ii+1)=x(i,j)
            end do
          end do
        end do
        do i=ii,min(ii+nb-1,n1)
          call stockham(work1(1,i-ii+1),work2,w2,n2)
        end do
        do j=1,n2
          do i=ii,min(ii+nb-1,n1)
            x(i,j)=work1(j,i-ii+1)*w(i,j)
          end do
        end do
      end do
      do jj=1,n2,nb
        do j=jj,min(jj+nb-1,n2)
          call stockham(x(1,j),work2,w1,n1)
        end do
        do i=1,n1
          do j=jj,min(jj+nb-1,n2)
            y(j,i)=x(i,j)
          end do
        end do
      end do
!
      deallocate(w1,w2,work1,work2)
      return
      end
      subroutine settbl(w,n)
      implicit real*8 (a-h,o-z)
      complex*16 w(n)
!
      ip=ilog2(n)
!
      j=1
      l=n
      do i=1,ip
        l=l/2
        call settbl0(w(j),l)
        j=j+l
      end do
      return
      end
      subroutine settbl0(w,l)
      implicit real*8 (a-h,o-z)
      complex*16 w(l)
!
      pi=4.0d0*atan(1.0d0)
      px=-2.0d0*pi/dble(2*l)
      do j=1,l
        wr=cos(px*dble(j-1))
        wi=sin(px*dble(j-1))
        w(j)=dcmplx(wr,wi)
      end do
      return
      end
      subroutine settbl2(w,n1,n2)
      implicit real*8 (a-h,o-z)
      complex*16 w(n1,n2)
!
      pi=4.0d0*atan(1.0d0)
      px=-2.0d0*pi/dble(n1*n2)
      do j=1,n2
        do i=1,n1
          wr=cos(px*dble(i-1)*dble(j-1))
          wi=sin(px*dble(i-1)*dble(j-1))
          w(i,j)=dcmplx(wr,wi)
        end do
      end do
      return
      end
      function ilog2(n)
!
      ilog2=0
      n2=n
   10 if (n2 .ge. 2) then
        n2=n2/2
        ilog2=ilog2+1
        go to 10
      end if
      return
      end
